#!/usr/bin/perl -w
use strict;
use Math::Trig;
use Getopt::Long;
use File::Path;
use File::Copy;
use Cwd 'abs_path';


#########################################################################check OS
my $system    = $^O;
my $pkgdir    = "";
my $libdir    = "";
my $decoydir  = "";
my $protname  = "";
my $model     = "";
my $datadir   = "";
my $javadir   = "";
my $usrname   = "";
my $runstyle  = "serial";

unless (lc($system) eq "linux")
{
    printf("Your operating system $system is unsupported at this time\n");
    printf("Currently only Linux is supported\n");
    exit();
}
GetOptions('pkgdir:s' => \$pkgdir, 'libdir:s' => \$libdir, 'protname:s' => \$protname, 'runstyle:s' => \$runstyle, 'model:s' => \$model, 'datadir:s' => \$datadir,  'decoydir:s' => \$decoydir, 'java_home:s' => \$javadir);

if(!$libdir || !$protname || !$datadir || !$model || !$pkgdir)
{
    print "\nResQ usage:\nrunResQ.pl arguments\n";
    print "====================\n";
    print "Mandatory arguments:\n";
    print "====================\n";
    print "-pkgdir I-TASSER_Suite_directory (e.g., /home/username/I-TASSER4.3)\n";
    print "-libdir I-TASSER_template_library_directory (e.g., /home/username/ITLIB)\n";
    print "-protname protein_name (e.g., test)\n";
    print "-model name_of_structure_file (e.g., model.pdb)\n";
    print "-datadir data_dir (where the structure file is located)\n";
    print "-java_home JAVA_HOME_path (where the program \"bin/java\" is located)\n\n";


    print "===================\n";
    print "Optional arguments:\n";
    print "===================\n";
    print "-runstyle   [parallel or serial] run scripts directly (serial, the default value) or submit LOMETS jobs with qsub (parallel)\n";
    print "-decoydir   where the related decoy structures (in PDB format) are located\n";

    exit();
}	 

my $fname=""; # in case the user provide a file name
($datadir, $fname)=&get_absolute_path($datadir);
($libdir, $fname)=&get_absolute_path($libdir);
($pkgdir, $fname)=&get_absolute_path($pkgdir);


if(!$usrname)
{
    $usrname=getlogin() || $ENV{'USER'};
}

 
print "\n\nYour setting for running ResQ is:\n";
print "-pkgdir    = $pkgdir\n";
print "-libdir    = $libdir\n";
print "-protname  = $protname\n";
print "-datadir   = $datadir\n";
print "-java_home = $javadir\n";
if($decoydir ne "")
{
    print "-decoydir   = $decoydir\n";
}

my $bindir     = "$pkgdir/ResQ/bin"; ### Where all programs are stored
if(!-s "$bindir/resqmod")
{
    print "$bindir/resqmod not exist. stop\n";
    exit;
}
if(!-s "$datadir/$model")
{
    print "$datadir/$model does not exist. stop\n";
    exit;
}


my $modelname=$model;
$modelname =~ s/.pdb$//;
my $inputdir="$datadir";
`mkdir -p $inputdir`;

my $tag        ="R_$protname\_$modelname"; # unique name for job
my $jobname    ="$tag.pl";
	  
my $walltime   ="walltime=20:00:00";        
my $errfile    ="$inputdir/err_$tag";
my $outfile    ="$inputdir/out_$tag";


  
my $mod = "";
$mod    = `cat $bindir/resqmod`;	    
$mod =~ s/\!USER\!/$usrname/mg;
$mod =~ s/\!BINDIR\!/$bindir/mg;
$mod =~ s/\!DATADIR\!/$datadir/mg;
$mod =~ s/\!TARGET\!/$protname/mg;
$mod =~ s/\!MODEL\!/$model/mg;
$mod =~ s/\!ITLIB\!/$libdir/mg;
$mod =~ s/\!PKGDIR\!/$pkgdir/mg;
$mod =~ s/\!STYLE\!/$runstyle/mg;
$mod =~ s/\!JAVA\!/$javadir/mg;
$mod =~ s/\!DECOYDIR\!/$decoydir/mg;


open(FH2,">$inputdir/$jobname");
print FH2 "$mod";
close(FH2);
`chmod a+x $inputdir/$jobname`;
 
if($runstyle eq "parallel")
{
    my $running= `qstat -u $usrname`;
    if($running !~ /$tag/)
    {
	my $bsub=`qsub -e $errfile -o $outfile -l $walltime -N $tag $inputdir/$jobname`;
	chomp($bsub);
	if(length $bsub ==0)
	{	    
	    print "$jobname was not submitted. Please check if qsub works properly\n";
	}
	else
	{
	    print "$jobname was submitted\n";
	}
	sleep(2);
    }   
}
else
{
    print "\nrun $inputdir/$jobname...\n";
    print `$inputdir/$jobname`; 
    print "ResQ done\n";
}	    

exit;



sub get_absolute_path
{
    my ($path)=@_;
    my $dir = "";
    my $fname= "";
    if($path ne "")
    {
        my $apath = abs_path($path); # for '..'        
        if (defined($apath))
        {
            $dir = $apath;
            if(-e $dir && !-d $dir) #it is not an existing directory, should be a file
            {
                $dir =~ s/(.*)\/.*$/$1/;
                $fname=$apath;
                $fname =~ s/^.*[\/\\]//;
            }
        }
    }
    return ($dir, $fname);
}


